@@ -206,7 +206,7 @@ setMethod("gsva", signature(expr="SummarizedExperiment", gset.idx.list="GeneSetC
     if (!annotation %in% names(assays(se)))
       stop(sprintf("Assay %s not found in the input SummarizedExperiment object.", annotation))
   }
-  expr <- assays(se)[[annotation]]
+  expr <- as.matrix(assays(se)[[annotation]])
 
   ## filter genes according to verious criteria,
   ## e.g., constant expression
@@ -296,7 +296,7 @@ setMethod("gsva", signature(expr="SummarizedExperiment", gset.idx.list="list"),
     if (!annotation %in% names(assays(se)))
       stop(sprintf("Assay %s not found in the input SummarizedExperiment object.", annotation))
   }
-  expr <- assays(se)[[annotation]]
+  expr <- as.matrix(assays(se)[[annotation]])
 
   ## filter genes according to verious criteria,
   ## e.g., constant expression
@@ -852,12 +852,12 @@ ssgsea <- function(X, geneSets, alpha=0.25, parallel.sz,
 
   n <- ncol(X)
   
-  if(is(X, "dgCMatrix")){
-    R <- t(sparseMatrixStats::colRanks(X, ties.method = "average"))
-    mode(R) <- "integer"
-  } else {
-    R <- apply(X, 2, function(x, p) as.integer(rank(x)))
-  }
+  print("Calculating ranks...")
+  
+  R <- t(sparseMatrixStats::colRanks(X, ties.method = "average"))
+  mode(R) <- "integer"
+
+  print("Calculating absolute values from ranks...")
   
   Ra <- abs(R)^alpha
   
@@ -872,9 +872,10 @@ ssgsea <- function(X, geneSets, alpha=0.25, parallel.sz,
   
   
   if (normalization) {
+    print("Normalizing...")
     ## normalize enrichment scores by using the entire data set, as indicated
     ## by Barbie et al., 2009, online methods, pg. 2
-    es <- apply(es, 2, function(x, es) x / (range(es)[2] - range(es)[1]), es)
+    es <- es[, 1:n, drop=FALSE] / (range(es)[2] - range(es)[1])
   }
   
   if (length(geneSets) == 1)
